﻿using System;
using System.Text;
using System.Collections.Generic;

namespace plexdata.CodePage
{
    internal class CharacterConverter
    {
        private static CharacterConverter theInstance = null;
        internal static CharacterConverter Instance
        {
            get
            {
                if (CharacterConverter.theInstance == null)
                {
                    CharacterConverter.theInstance = new CharacterConverter();
                }

                return CharacterConverter.theInstance;
            }
        }

        public enum MappingTypes { Unmapped, Mapped, Controls }

        private MappingTypes mappingTyp = MappingTypes.Mapped;

        private Dictionary<char, string> mapping = null;
        private Dictionary<char, string> alternative = null;

        private CharacterConverter()
        {
            #region Initialize non-displayable character mappings.

            this.mapping = new Dictionary<char, string>();

            this.mapping.Add('\u0000', "(NUL)");
            this.mapping.Add('\u0001', "(SOH)");
            this.mapping.Add('\u0002', "(STX)");
            this.mapping.Add('\u0003', "(ETX)");
            this.mapping.Add('\u0004', "(EOT)");
            this.mapping.Add('\u0005', "(ENQ)");
            this.mapping.Add('\u0006', "(ACK)");
            this.mapping.Add('\u0007', "(BEL)");
            this.mapping.Add('\u0008', "(BS)");
            this.mapping.Add('\u0009', "(HT)");
            this.mapping.Add('\u000A', "(LF)");
            this.mapping.Add('\u000B', "(VT)");
            this.mapping.Add('\u000C', "(FF)");
            this.mapping.Add('\u000D', "(CR)");
            this.mapping.Add('\u000E', "(SO)");
            this.mapping.Add('\u000F', "(SI)");
            this.mapping.Add('\u0010', "(DLE)");
            this.mapping.Add('\u0011', "(DC1)");
            this.mapping.Add('\u0012', "(DC2)");
            this.mapping.Add('\u0013', "(DC3)");
            this.mapping.Add('\u0014', "(DC4)");
            this.mapping.Add('\u0015', "(NAK)");
            this.mapping.Add('\u0016', "(SYN)");
            this.mapping.Add('\u0017', "(ETB)");
            this.mapping.Add('\u0018', "(CAN)");
            this.mapping.Add('\u0019', "(EM)");
            this.mapping.Add('\u001A', "(SUB)");
            this.mapping.Add('\u001B', "(ESC)");
            this.mapping.Add('\u001C', "(FS)");
            this.mapping.Add('\u001D', "(GS)");
            this.mapping.Add('\u001E', "(RS)");
            this.mapping.Add('\u001F', "(US)");
            this.mapping.Add('\u0020', "<space>");
            this.mapping.Add('\u007F', "(DEL)");

            #endregion // Initialize non-displayable character mappings.

            #region Initialize alternative character mappings.

            this.alternative = new Dictionary<char, string>();

            this.alternative.Add('\u0000', "\\0");
            this.alternative.Add('\u0007', "\\a");
            this.alternative.Add('\u0008', "\\b");
            this.alternative.Add('\u0009', "\\t");
            this.alternative.Add('\u000A', "\\n");
            this.alternative.Add('\u000B', "\\v");
            this.alternative.Add('\u000C', "\\f");
            this.alternative.Add('\u000D', "\\r");
            this.alternative.Add('\u0020', "<space>");

            #endregion // Initialize alternative character mappings.
        }

        public MappingTypes MappingType
        {
            get { return this.mappingTyp; }
            set { this.mappingTyp = value; }
        }

        internal string Encode(char input, Encoding dstEncoding)
        {
            return this.Encode(Convert.ToInt32(input), dstEncoding);
        }

        internal string Encode(char input, Encoding srcEncoding, Encoding dstEncoding)
        {
            char[] helper = new char[] { input };
            byte[] buffer = Encoding.Convert(srcEncoding, dstEncoding, srcEncoding.GetBytes(helper));
            return this.CharacterMapping(input, buffer, dstEncoding);
        }

        internal string Encode(int input, Encoding dstEncoding)
        {
            return this.Encode(char.ConvertFromUtf32(input)[0], Encoding.UTF32, dstEncoding);
        }

        internal string Encode(int input, Encoding srcEncoding, Encoding dstEncoding)
        {
            return this.Encode(Convert.ToChar(input), srcEncoding, dstEncoding);
        }

        private string CharacterMapping(char input, byte[] buffer, Encoding encoding)
        {
            string result = "";

            if (this.mappingTyp == MappingTypes.Mapped)
            {
                if (!this.mapping.TryGetValue(input, out result))
                {
                    result = encoding.GetString(buffer);
                }
            }
            else if (this.mappingTyp == MappingTypes.Controls)
            {
                if (!this.alternative.TryGetValue(input, out result))
                {
                    result = encoding.GetString(buffer);
                }
            }
            else if (this.mappingTyp == MappingTypes.Unmapped)
            {
                result = encoding.GetString(buffer);
            }
        
            return result;
        }
    }
}
